@@ -18,7 +18,7 @@ docker run \
   -e ENABLE_ARCH_i686="$i686" \
   -e ENABLE_ARCH_x86_64="$x86_64" \
   -e ENABLE_TEST=true \
-  -e EXTRA_CMAKE_FLAGS="-DBOOTSTRAP_DAEMON=OFF -DMIN_LOGGER_LEVEL=DEBUG -DTEST_TIMEOUT_SECONDS=90 -DAUTOTEST=ON" \
+  -e EXTRA_CMAKE_FLAGS="-DBOOTSTRAP_DAEMON=OFF -DMIN_LOGGER_LEVEL=DEBUG -DTEST_TIMEOUT_SECONDS=90 -DAUTOTEST=ON -DUSE_IPV6=OFF" \
   -e CMAKE_C_FLAGS="$C_FLAGS" \
   -e CMAKE_CXX_FLAGS="$CXX_FLAGS" \
   -e CMAKE_EXE_LINKER_FLAGS="$LD_FLAGS" \
@@ -26,4 +26,6 @@ docker run \
   -v "$PWD:/toxcore" \
   -v "$PWD/result:/prefix" \
   --rm \
+  -t \
+  --pull never \
   "toxchat/windows:$WINDOWS_ARCH"
