@@ -5,16 +5,16 @@
 add_ld_flag -Wl,-z,defs
 
 # Make compilation error on a warning
-add_flag -Werror
+add_flag -Werror -Wno-unsafe-buffer-usage
 
 # Coverage flags.
-add_flag --coverage
+add_flag -fprofile-instr-generate -fcoverage-mapping
 
 # Optimisation, but keep stack traces useful.
 add_c_flag -fno-inline -fno-omit-frame-pointer
 
 # Show useful stack traces on crash.
-add_flag -fsanitize=undefined -fno-sanitize-recover=all
+add_flag -fsanitize=undefined -fno-sanitize-recover=all -D_DEBUG
 
 # In test code (_test.cc and libgtest), throw away all debug information.
 # We only care about stack frames inside toxcore (which is C). Without this,
