@@ -3,12 +3,9 @@
 // This test triggers a different code path than if we only allocate a single
 // conference. This is the simplest test possible that triggers it.
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "../testing/misc_tools.h"
 #include "../toxcore/tox.h"
+#include "auto_test_support.h"
 #include "check_compat.h"
 
 int main(void)
