@@ -37,11 +37,11 @@ static void do_onion(Mono_Time *mono_time, Onion *onion)
 static int handled_test_1;
 static int handle_test_1(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
-    Onion *onion = (Onion *)object;
+    const Onion *onion = (const Onion *)object;
 
     const char req_message[] = "Install Gentoo";
     uint8_t req_packet[1 + sizeof(req_message)];
-    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST_OLD;
+    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST;
     memcpy(req_packet + 1, req_message, sizeof(req_message));
 
     if (memcmp(packet, req_packet, sizeof(req_packet)) != 0) {
@@ -50,10 +50,10 @@ static int handle_test_1(void *object, const IP_Port *source, const uint8_t *pac
 
     const char res_message[] = "install gentoo";
     uint8_t res_packet[1 + sizeof(res_message)];
-    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE_OLD;
+    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE;
     memcpy(res_packet + 1, res_message, sizeof(res_message));
 
-    if (send_onion_response(onion->net, source, res_packet, sizeof(res_packet),
+    if (send_onion_response(onion->log, onion->net, source, res_packet, sizeof(res_packet),
                             packet + sizeof(res_packet)) == -1) {
         return 1;
     }
@@ -67,7 +67,7 @@ static int handle_test_2(void *object, const IP_Port *source, const uint8_t *pac
 {
     const char res_message[] = "install gentoo";
     uint8_t res_packet[1 + sizeof(res_message)];
-    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE_OLD;
+    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE;
     memcpy(res_packet + 1, res_message, sizeof(res_message));
 
     if (length != sizeof(res_packet)) {
@@ -99,14 +99,48 @@ static uint8_t test_3_pub_key[CRYPTO_PUBLIC_KEY_SIZE];
 static uint8_t test_3_ping_id[CRYPTO_SHA256_SIZE];
 static int handle_test_3(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
-    Onion *onion = (Onion *)object;
+    const Onion *onion = (const Onion *)object;
 
-    if (length != (1 + CRYPTO_NONCE_SIZE + ONION_ANNOUNCE_SENDBACK_DATA_LENGTH + 1 + CRYPTO_SHA256_SIZE +
-                   CRYPTO_MAC_SIZE)) {
+    if (length < ONION_ANNOUNCE_RESPONSE_MIN_SIZE || length > ONION_ANNOUNCE_RESPONSE_MAX_SIZE) {
         return 1;
     }
 
-    uint8_t plain[1 + CRYPTO_SHA256_SIZE];
+    uint8_t plain[2 + CRYPTO_SHA256_SIZE];
+#if 0
+    print_client_id(packet, length);
+#endif
+    int len = decrypt_data(test_3_pub_key, dht_get_self_secret_key(onion->dht),
+                           packet + 1 + ONION_ANNOUNCE_SENDBACK_DATA_LENGTH,
+                           packet + 1 + ONION_ANNOUNCE_SENDBACK_DATA_LENGTH + CRYPTO_NONCE_SIZE,
+                           2 + CRYPTO_SHA256_SIZE + CRYPTO_MAC_SIZE, plain);
+
+    if (len == -1) {
+        return 1;
+    }
+
+    if (memcmp(packet + 1, sb_data, ONION_ANNOUNCE_SENDBACK_DATA_LENGTH) != 0) {
+        return 1;
+    }
+
+    memcpy(test_3_ping_id, plain + 1, CRYPTO_SHA256_SIZE);
+#if 0
+    print_client_id(test_3_ping_id, sizeof(test_3_ping_id));
+#endif
+    handled_test_3 = 1;
+    return 0;
+}
+
+/* TODO: DEPRECATE */
+static int handle_test_3_old(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length,
+                             void *userdata)
+{
+    const Onion *onion = (const Onion *)object;
+
+    if (length < ONION_ANNOUNCE_RESPONSE_MIN_SIZE || length > ONION_ANNOUNCE_RESPONSE_MAX_SIZE) {
+        return 1;
+    }
+
+    uint8_t plain[2 + CRYPTO_SHA256_SIZE];
 #if 0
     print_client_id(packet, length);
 #endif
@@ -119,7 +153,6 @@ static int handle_test_3(void *object, const IP_Port *source, const uint8_t *pac
         return 1;
     }
 
-
     if (memcmp(packet + 1, sb_data, ONION_ANNOUNCE_SENDBACK_DATA_LENGTH) != 0) {
         return 1;
     }
@@ -136,7 +169,7 @@ static uint8_t nonce[CRYPTO_NONCE_SIZE];
 static int handled_test_4;
 static int handle_test_4(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
-    Onion *onion = (Onion *)object;
+    const Onion *onion = (const Onion *)object;
 
     if (length != (1 + CRYPTO_NONCE_SIZE + CRYPTO_PUBLIC_KEY_SIZE + sizeof("Install gentoo") +
                    CRYPTO_MAC_SIZE)) {
@@ -180,31 +213,34 @@ static void send_onion_packet(const Networking_Core *net, const Random *rng, con
 /** Initialize networking.
  * Added for reverse compatibility with old new_networking calls.
  */
-static Networking_Core *new_networking(const Logger *log, const Network *ns, const IP *ip, uint16_t port)
+static Networking_Core *new_networking(const Logger *log, const Memory *mem, const Network *ns, const IP *ip, uint16_t port)
 {
-    return new_networking_ex(log, ns, ip, port, port + (TOX_PORTRANGE_TO - TOX_PORTRANGE_FROM), nullptr);
+    return new_networking_ex(log, mem, ns, ip, port, port + (TOX_PORTRANGE_TO - TOX_PORTRANGE_FROM), nullptr);
 }
 
 static void test_basic(void)
 {
     uint32_t index[] = { 1, 2, 3 };
-    const Network *ns = system_network();
+    const Network *ns = os_network();
+    ck_assert(ns != nullptr);
+    const Memory *mem = os_memory();
+    ck_assert(mem != nullptr);
+    const Random *rng = os_random();
+    ck_assert(rng != nullptr);
 
     Logger *log1 = logger_new();
-    logger_callback_log(log1, (logger_cb *)print_debug_log, nullptr, &index[0]);
+    logger_callback_log(log1, print_debug_logger, nullptr, &index[0]);
     Logger *log2 = logger_new();
-    logger_callback_log(log2, (logger_cb *)print_debug_log, nullptr, &index[1]);
+    logger_callback_log(log2, print_debug_logger, nullptr, &index[1]);
 
-    const Random *rng = system_random();
-    ck_assert(rng != nullptr);
-    Mono_Time *mono_time1 = mono_time_new(nullptr, nullptr);
-    Mono_Time *mono_time2 = mono_time_new(nullptr, nullptr);
+    Mono_Time *mono_time1 = mono_time_new(mem, nullptr, nullptr);
+    Mono_Time *mono_time2 = mono_time_new(mem, nullptr, nullptr);
 
     IP ip = get_loopback();
-    Onion *onion1 = new_onion(log1, mono_time1, rng, new_dht(log1, rng, ns, mono_time1, new_networking(log1, ns, &ip, 36567), true, false));
-    Onion *onion2 = new_onion(log2, mono_time2, rng, new_dht(log2, rng, ns, mono_time2, new_networking(log2, ns, &ip, 36568), true, false));
+    Onion *onion1 = new_onion(log1, mem, mono_time1, rng, new_dht(log1, mem, rng, ns, mono_time1, new_networking(log1, mem, ns, &ip, 36567), true, false));
+    Onion *onion2 = new_onion(log2, mem, mono_time2, rng, new_dht(log2, mem, rng, ns, mono_time2, new_networking(log2, mem, ns, &ip, 36568), true, false));
     ck_assert_msg((onion1 != nullptr) && (onion2 != nullptr), "Onion failed initializing.");
-    networking_registerhandler(onion2->net, NET_PACKET_ANNOUNCE_REQUEST_OLD, &handle_test_1, onion2);
+    networking_registerhandler(onion2->net, NET_PACKET_ANNOUNCE_REQUEST, &handle_test_1, onion2);
 
     IP_Port on1 = {ip, net_port(onion1->net)};
     Node_format n1;
@@ -218,7 +254,7 @@ static void test_basic(void)
 
     const char req_message[] = "Install Gentoo";
     uint8_t req_packet[1 + sizeof(req_message)];
-    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST_OLD;
+    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST;
     memcpy(req_packet + 1, req_message, sizeof(req_message));
 
     Node_format nodes[4];
@@ -237,7 +273,7 @@ static void test_basic(void)
         do_onion(mono_time2, onion2);
     } while (handled_test_1 == 0);
 
-    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE_OLD, &handle_test_2, onion1);
+    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE, &handle_test_2, onion1);
     handled_test_2 = 0;
 
     do {
@@ -245,16 +281,17 @@ static void test_basic(void)
         do_onion(mono_time2, onion2);
     } while (handled_test_2 == 0);
 
-    Onion_Announce *onion1_a = new_onion_announce(log1, rng, mono_time1, onion1->dht);
-    Onion_Announce *onion2_a = new_onion_announce(log2, rng, mono_time2, onion2->dht);
-    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE_OLD, &handle_test_3, onion1);
+    Onion_Announce *onion1_a = new_onion_announce(log1, mem, rng, mono_time1, onion1->dht);
+    Onion_Announce *onion2_a = new_onion_announce(log2, mem, rng, mono_time2, onion2->dht);
+    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE, &handle_test_3, onion1);
+    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE_OLD, &handle_test_3_old, onion1);
     ck_assert_msg((onion1_a != nullptr) && (onion2_a != nullptr), "Onion_Announce failed initializing.");
-    uint8_t zeroes[64] = {0};
+    const uint8_t zeroes[64] = {0};
     random_bytes(rng, sb_data, sizeof(sb_data));
     uint64_t s;
     memcpy(&s, sb_data, sizeof(uint64_t));
     memcpy(test_3_pub_key, nodes[3].public_key, CRYPTO_PUBLIC_KEY_SIZE);
-    int ret = send_announce_request(onion1->net, rng, &path, &nodes[3],
+    int ret = send_announce_request(log1, onion1->net, rng, &path, &nodes[3],
                                     dht_get_self_public_key(onion1->dht),
                                     dht_get_self_secret_key(onion1->dht),
                                     zeroes,
@@ -269,12 +306,14 @@ static void test_basic(void)
         c_sleep(50);
     } while (handled_test_3 == 0);
 
+    printf("test 3 complete\n");
+
     random_bytes(rng, sb_data, sizeof(sb_data));
     memcpy(&s, sb_data, sizeof(uint64_t));
     memcpy(onion_announce_entry_public_key(onion2_a, 1), dht_get_self_public_key(onion2->dht), CRYPTO_PUBLIC_KEY_SIZE);
     onion_announce_entry_set_time(onion2_a, 1, mono_time_get(mono_time2));
     networking_registerhandler(onion1->net, NET_PACKET_ONION_DATA_RESPONSE, &handle_test_4, onion1);
-    send_announce_request(onion1->net, rng, &path, &nodes[3],
+    send_announce_request(log1, onion1->net, rng, &path, &nodes[3],
                           dht_get_self_public_key(onion1->dht),
                           dht_get_self_secret_key(onion1->dht),
                           test_3_ping_id,
@@ -291,15 +330,15 @@ static void test_basic(void)
 
     c_sleep(1000);
     Logger *log3 = logger_new();
-    logger_callback_log(log3, (logger_cb *)print_debug_log, nullptr, &index[2]);
+    logger_callback_log(log3, print_debug_logger, nullptr, &index[2]);
 
-    Mono_Time *mono_time3 = mono_time_new(nullptr, nullptr);
+    Mono_Time *mono_time3 = mono_time_new(mem, nullptr, nullptr);
 
-    Onion *onion3 = new_onion(log3, mono_time3, rng, new_dht(log3, rng, ns, mono_time3, new_networking(log3, ns, &ip, 36569), true, false));
+    Onion *onion3 = new_onion(log3, mem, mono_time3, rng, new_dht(log3, mem, rng, ns, mono_time3, new_networking(log3, mem, ns, &ip, 36569), true, false));
     ck_assert_msg((onion3 != nullptr), "Onion failed initializing.");
 
     random_nonce(rng, nonce);
-    ret = send_data_request(onion3->net, rng, &path, &nodes[3].ip_port,
+    ret = send_data_request(log3, onion3->net, rng, &path, &nodes[3].ip_port,
                             dht_get_self_public_key(onion1->dht),
                             dht_get_self_public_key(onion1->dht),
                             nonce, (const uint8_t *)"Install gentoo", sizeof("Install gentoo"));
@@ -312,6 +351,8 @@ static void test_basic(void)
         c_sleep(50);
     } while (handled_test_4 == 0);
 
+    printf("test 4 complete\n");
+
     kill_onion_announce(onion2_a);
     kill_onion_announce(onion1_a);
 
@@ -323,7 +364,7 @@ static void test_basic(void)
         kill_onion(onion);
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(mono_time3);
+        mono_time_free(mem, mono_time3);
         logger_kill(log3);
     }
 
@@ -335,7 +376,7 @@ static void test_basic(void)
         kill_onion(onion);
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(mono_time2);
+        mono_time_free(mem, mono_time2);
         logger_kill(log2);
     }
 
@@ -347,7 +388,7 @@ static void test_basic(void)
         kill_onion(onion);
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(mono_time1);
+        mono_time_free(mem, mono_time1);
         logger_kill(log1);
     }
 }
@@ -360,11 +401,11 @@ typedef struct {
     Onion_Client *onion_c;
 } Onions;
 
-static Onions *new_onions(const Random *rng, uint16_t port, uint32_t *index)
+static Onions *new_onions(const Memory *mem, const Random *rng, uint16_t port, uint32_t *index)
 {
     IP ip = get_loopback();
     ip.ip.v6.uint8[15] = 1;
-    const Network *ns = system_network();
+    const Network *ns = os_network();
     Onions *on = (Onions *)malloc(sizeof(Onions));
 
     if (!on) {
@@ -378,9 +419,9 @@ static Onions *new_onions(const Random *rng, uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    logger_callback_log(on->log, (logger_cb *)print_debug_log, nullptr, index);
+    logger_callback_log(on->log, print_debug_logger, nullptr, index);
 
-    on->mono_time = mono_time_new(nullptr, nullptr);
+    on->mono_time = mono_time_new(mem, nullptr, nullptr);
 
     if (!on->mono_time) {
         logger_kill(on->log);
@@ -388,57 +429,57 @@ static Onions *new_onions(const Random *rng, uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    Networking_Core *net = new_networking(on->log, ns, &ip, port);
+    Networking_Core *net = new_networking(on->log, mem, ns, &ip, port);
 
     if (!net) {
-        mono_time_free(on->mono_time);
+        mono_time_free(mem, on->mono_time);
         logger_kill(on->log);
         free(on);
         return nullptr;
     }
 
-    DHT *dht = new_dht(on->log, rng, ns, on->mono_time, net, true, false);
+    DHT *dht = new_dht(on->log, mem, rng, ns, on->mono_time, net, true, false);
 
     if (!dht) {
         kill_networking(net);
-        mono_time_free(on->mono_time);
+        mono_time_free(mem, on->mono_time);
         logger_kill(on->log);
         free(on);
         return nullptr;
     }
 
-    on->onion = new_onion(on->log, on->mono_time, rng, dht);
+    on->onion = new_onion(on->log, mem, on->mono_time, rng, dht);
 
     if (!on->onion) {
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(on->mono_time);
+        mono_time_free(mem, on->mono_time);
         logger_kill(on->log);
         free(on);
         return nullptr;
     }
 
-    on->onion_a = new_onion_announce(on->log, rng, on->mono_time, dht);
+    on->onion_a = new_onion_announce(on->log, mem, rng, on->mono_time, dht);
 
     if (!on->onion_a) {
         kill_onion(on->onion);
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(on->mono_time);
+        mono_time_free(mem, on->mono_time);
         logger_kill(on->log);
         free(on);
         return nullptr;
     }
 
     TCP_Proxy_Info inf = {{{{0}}}};
-    on->onion_c = new_onion_client(on->log, rng, on->mono_time, new_net_crypto(on->log, rng, ns, on->mono_time, dht, &inf));
+    on->onion_c = new_onion_client(on->log, mem, rng, on->mono_time, new_net_crypto(on->log, mem, rng, ns, on->mono_time, dht, &inf));
 
     if (!on->onion_c) {
         kill_onion_announce(on->onion_a);
         kill_onion(on->onion);
         kill_dht(dht);
         kill_networking(net);
-        mono_time_free(on->mono_time);
+        mono_time_free(mem, on->mono_time);
         logger_kill(on->log);
         free(on);
         return nullptr;
@@ -456,7 +497,7 @@ static void do_onions(Onions *on)
     do_onion_client(on->onion_c);
 }
 
-static void kill_onions(Onions *on)
+static void kill_onions(const Memory *mem, Onions *on)
 {
     Networking_Core *net = dht_get_net(on->onion->dht);
     DHT *dht = on->onion->dht;
@@ -467,7 +508,7 @@ static void kill_onions(Onions *on)
     kill_net_crypto(c);
     kill_dht(dht);
     kill_networking(net);
-    mono_time_free(on->mono_time);
+    mono_time_free(mem, on->mono_time);
     logger_kill(on->log);
     free(on);
 }
@@ -500,10 +541,10 @@ static void dht_pk_callback(void *object, int32_t number, const uint8_t *dht_pub
 {
     if ((NUM_FIRST == number && !first) || (NUM_LAST == number && !last)) {
         Onions *on = (Onions *)object;
-        uint16_t count = 0;
-        int ret = dht_addfriend(on->onion->dht, dht_public_key, &dht_ip_callback, object, number, &count);
+        uint32_t token = 0;
+        int ret = dht_addfriend(on->onion->dht, dht_public_key, &dht_ip_callback, object, number, &token);
         ck_assert_msg(ret == 0, "dht_addfriend() did not return 0");
-        ck_assert_msg(count == 1, "Count not 1, count is %u", count);
+        ck_assert_msg(token == 1, "Count not 1, count is %u", token);
 
         if (NUM_FIRST == number && !first) {
             first = 1;
@@ -531,21 +572,22 @@ static void dht_pk_callback(void *object, int32_t number, const uint8_t *dht_pub
 
 static void test_announce(void)
 {
-    uint32_t i, j;
     uint32_t index[NUM_ONIONS];
     Onions *onions[NUM_ONIONS];
-    const Random *rng = system_random();
+    const Random *rng = os_random();
     ck_assert(rng != nullptr);
+    const Memory *mem = os_memory();
+    ck_assert(mem != nullptr);
 
-    for (i = 0; i < NUM_ONIONS; ++i) {
+    for (uint32_t i = 0; i < NUM_ONIONS; ++i) {
         index[i] = i + 1;
-        onions[i] = new_onions(rng, i + 36655, &index[i]);
+        onions[i] = new_onions(mem, rng, i + 36655, &index[i]);
         ck_assert_msg(onions[i] != nullptr, "Failed to create onions. %u", i);
     }
 
     IP ip = get_loopback();
 
-    for (i = 3; i < NUM_ONIONS; ++i) {
+    for (uint32_t i = 3; i < NUM_ONIONS; ++i) {
         IP_Port ip_port = {ip, net_port(onions[i - 1]->onion->net)};
         dht_bootstrap(onions[i]->onion->dht, &ip_port, dht_get_self_public_key(onions[i - 1]->onion->dht));
         IP_Port ip_port1 = {ip, net_port(onions[i - 2]->onion->net)};
@@ -559,7 +601,7 @@ static void test_announce(void)
     do {
         connected = 0;
 
-        for (i = 0; i < NUM_ONIONS; ++i) {
+        for (uint32_t i = 0; i < NUM_ONIONS; ++i) {
             do_onions(onions[i]);
             connected += dht_isconnected(onions[i]->onion->dht);
         }
@@ -569,8 +611,8 @@ static void test_announce(void)
 
     printf("connected\n");
 
-    for (i = 0; i < 25 * 2; ++i) {
-        for (j = 0; j < NUM_ONIONS; ++j) {
+    for (uint32_t i = 0; i < 25 * 2; ++i) {
+        for (uint32_t j = 0; j < NUM_ONIONS; ++j) {
             do_onions(onions[j]);
         }
 
@@ -592,7 +634,7 @@ static void test_announce(void)
     IP_Port ip_port;
 
     do {
-        for (i = 0; i < NUM_ONIONS; ++i) {
+        for (uint32_t i = 0; i < NUM_ONIONS; ++i) {
             do_onions(onions[i]);
         }
 
@@ -602,7 +644,7 @@ static void test_announce(void)
     printf("Waiting for ips\n");
 
     do {
-        for (i = 0; i < NUM_ONIONS; ++i) {
+        for (uint32_t i = 0; i < NUM_ONIONS; ++i) {
             do_onions(onions[i]);
         }
 
@@ -612,8 +654,8 @@ static void test_announce(void)
     onion_getfriendip(onions[NUM_LAST]->onion_c, frnum, &ip_port);
     ck_assert_msg(ip_port.port == net_port(onions[NUM_FIRST]->onion->net), "Port in returned ip not correct.");
 
-    for (i = 0; i < NUM_ONIONS; ++i) {
-        kill_onions(onions[i]);
+    for (uint32_t i = 0; i < NUM_ONIONS; ++i) {
+        kill_onions(mem, onions[i]);
     }
 }
 
