@@ -17,14 +17,14 @@
  *
  * Important: You are responsible for freeing `pid_file_path` and `keys_file_path`
  *            also, iff `tcp_relay_ports_count` > 0, then you are responsible for freeing `tcp_relay_ports`
- *            and also `motd` iff `enable_motd` is set.
+ *            and also `motd` iff `enable_motd` is true.
  *
  * @return true on success,
  *         false on failure, doesn't modify any data pointed by arguments.
  */
 bool get_general_config(const char *cfg_file_path, char **pid_file_path, char **keys_file_path, int *port,
-                        int *enable_ipv6, int *enable_ipv4_fallback, int *enable_lan_discovery, int *enable_tcp_relay,
-                        uint16_t **tcp_relay_ports, int *tcp_relay_port_count, int *enable_motd, char **motd);
+                        bool *enable_ipv6, bool *enable_ipv4_fallback, bool *enable_lan_discovery, bool *enable_tcp_relay,
+                        uint16_t **tcp_relay_ports, int *tcp_relay_port_count, bool *enable_motd, char **motd);
 
 /**
  * Bootstraps off nodes listed in the config file.
