@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-3.0-or-later
- * Copyright © 2016-2018 The TokTok team.
+ * Copyright © 2016-2024 The TokTok team.
  * Copyright © 2014-2016 Tox project.
  */
 
@@ -23,16 +23,24 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 // toxcore
-#include "../../../toxcore/tox.h"
+#include "../../../toxcore/DHT.h"
 #include "../../../toxcore/LAN_discovery.h"
 #include "../../../toxcore/TCP_server.h"
 #include "../../../toxcore/announce.h"
+#include "../../../toxcore/ccompat.h"
+#include "../../../toxcore/crypto_core.h"
+#include "../../../toxcore/forwarding.h"
+#include "../../../toxcore/group_announce.h"
+#include "../../../toxcore/group_onion_announce.h"
 #include "../../../toxcore/logger.h"
+#include "../../../toxcore/mem.h"
 #include "../../../toxcore/mono_time.h"
+#include "../../../toxcore/network.h"
+#include "../../../toxcore/onion.h"
 #include "../../../toxcore/onion_announce.h"
-#include "../../../toxcore/util.h"
 
 // misc
 #include "../../bootstrap_node_packets.h"
@@ -42,7 +50,6 @@
 #include "global.h"
 #include "log.h"
 
-
 static void sleep_milliseconds(uint32_t ms)
 {
     struct timespec req;
@@ -53,24 +60,23 @@ static void sleep_milliseconds(uint32_t ms)
 
 // Uses the already existing key or creates one if it didn't exist
 //
-// returns 1 on success
-//         0 on failure - no keys were read or stored
+// returns true on success
+//         false on failure - no keys were read or stored
 
-static int manage_keys(DHT *dht, char *keys_file_path)
+static bool manage_keys(DHT *dht, const char *keys_file_path)
 {
     enum { KEYS_SIZE = CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_SECRET_KEY_SIZE };
     uint8_t keys[KEYS_SIZE];
-    FILE *keys_file;
 
     // Check if file exits, proceed to open and load keys
-    keys_file = fopen(keys_file_path, "rb");
+    FILE *keys_file = fopen(keys_file_path, "rb");
 
     if (keys_file != nullptr) {
         const size_t read_size = fread(keys, sizeof(uint8_t), KEYS_SIZE, keys_file);
 
         if (read_size != KEYS_SIZE) {
             fclose(keys_file);
-            return 0;
+            return false;
         }
 
         dht_set_self_public_key(dht, keys);
@@ -82,21 +88,21 @@ static int manage_keys(DHT *dht, char *keys_file_path)
 
         keys_file = fopen(keys_file_path, "wb");
 
-        if (!keys_file) {
-            return 0;
+        if (keys_file == nullptr) {
+            return false;
         }
 
         const size_t write_size = fwrite(keys, sizeof(uint8_t), KEYS_SIZE, keys_file);
 
         if (write_size != KEYS_SIZE) {
             fclose(keys_file);
-            return 0;
+            return false;
         }
     }
 
     fclose(keys_file);
 
-    return 1;
+    return true;
 }
 
 // Prints public key
@@ -116,12 +122,12 @@ static void print_public_key(const uint8_t *public_key)
 // Demonizes the process, appending PID to the PID file and closing file descriptors based on log backend
 // Terminates the application if the daemonization fails.
 
-static void daemonize(LOG_BACKEND log_backend, char *pid_file_path)
+static Cli_Status daemonize(LOG_BACKEND log_backend, char *pid_file_path)
 {
     // Check if the PID file exists
-    FILE *pid_file;
+    FILE *pid_file = fopen(pid_file_path, "r");
 
-    if ((pid_file = fopen(pid_file_path, "r"))) {
+    if (pid_file != nullptr) {
         log_write(LOG_LEVEL_WARNING, "Another instance of the daemon is already running, PID file %s exists.\n", pid_file_path);
         fclose(pid_file);
     }
@@ -131,7 +137,7 @@ static void daemonize(LOG_BACKEND log_backend, char *pid_file_path)
 
     if (pid_file == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't open the PID file for writing: %s. Exiting.\n", pid_file_path);
-        exit(1);
+        return CLI_STATUS_ERROR;
     }
 
     // Fork off from the parent process
@@ -141,27 +147,26 @@ static void daemonize(LOG_BACKEND log_backend, char *pid_file_path)
         fprintf(pid_file, "%d", pid);
         fclose(pid_file);
         log_write(LOG_LEVEL_INFO, "Forked successfully: PID: %d.\n", pid);
-        exit(0);
+        return CLI_STATUS_DONE;
     } else {
         fclose(pid_file);
     }
 
     if (pid < 0) {
         log_write(LOG_LEVEL_ERROR, "Forking failed. Exiting.\n");
-        exit(1);
+        return CLI_STATUS_ERROR;
     }
 
     // Create a new SID for the child process
     if (setsid() < 0) {
         log_write(LOG_LEVEL_ERROR, "SID creation failure. Exiting.\n");
-        exit(1);
+        return CLI_STATUS_ERROR;
     }
 
-
     // Change the current working directory
     if ((chdir("/")) < 0) {
         log_write(LOG_LEVEL_ERROR, "Couldn't change working directory to '/'. Exiting.\n");
-        exit(1);
+        return CLI_STATUS_ERROR;
     }
 
     // Go quiet
@@ -170,42 +175,35 @@ static void daemonize(LOG_BACKEND log_backend, char *pid_file_path)
         close(STDIN_FILENO);
         close(STDERR_FILENO);
     }
+
+    return CLI_STATUS_OK;
 }
 
 // Logs toxcore logger message using our logger facility
 
+static LOG_LEVEL logger_level_to_log_level(Logger_Level level)
+{
+    switch (level) {
+        case LOGGER_LEVEL_TRACE:
+        case LOGGER_LEVEL_DEBUG:
+        case LOGGER_LEVEL_INFO:
+            return LOG_LEVEL_INFO;
+
+        case LOGGER_LEVEL_WARNING:
+            return LOG_LEVEL_WARNING;
+
+        case LOGGER_LEVEL_ERROR:
+            return LOG_LEVEL_ERROR;
+
+        default:
+            return LOG_LEVEL_INFO;
+    }
+}
+
 static void toxcore_logger_callback(void *context, Logger_Level level, const char *file, int line,
                                     const char *func, const char *message, void *userdata)
 {
-    LOG_LEVEL log_level;
-
-    switch (level) {
-        case LOGGER_LEVEL_TRACE:
-            log_level = LOG_LEVEL_INFO;
-            break;
-
-        case LOGGER_LEVEL_DEBUG:
-            log_level = LOG_LEVEL_INFO;
-            break;
-
-        case LOGGER_LEVEL_INFO:
-            log_level = LOG_LEVEL_INFO;
-            break;
-
-        case LOGGER_LEVEL_WARNING:
-            log_level = LOG_LEVEL_WARNING;
-            break;
-
-        case LOGGER_LEVEL_ERROR:
-            log_level = LOG_LEVEL_ERROR;
-            break;
-
-        default:
-            log_level = LOG_LEVEL_INFO;
-            break;
-    }
-
-    log_write(log_level, "%s:%d(%s) %s\n", file, line, func, message);
+    log_write(logger_level_to_log_level(level), "%s:%d(%s) %s\n", file, line, func, message);
 }
 
 static volatile sig_atomic_t caught_signal = 0;
@@ -219,14 +217,20 @@ int main(int argc, char *argv[])
 {
     umask(077);
     char *cfg_file_path = nullptr;
-    LOG_BACKEND log_backend;
-    bool run_in_foreground;
+    bool run_in_foreground = false;
 
-    // choose backend for printing command line argument parsing output based on whether the daemon is being run from a terminal
-    log_backend = isatty(STDOUT_FILENO) ? LOG_BACKEND_STDOUT : LOG_BACKEND_SYSLOG;
+    // Choose backend for printing command line argument parsing output based on whether the daemon is being run from a terminal
+    LOG_BACKEND log_backend = isatty(STDOUT_FILENO) != 0 ? LOG_BACKEND_STDOUT : LOG_BACKEND_SYSLOG;
 
     log_open(log_backend);
-    handle_command_line_arguments(argc, argv, &cfg_file_path, &log_backend, &run_in_foreground);
+    switch (handle_command_line_arguments(argc, argv, &cfg_file_path, &log_backend, &run_in_foreground)) {
+        case CLI_STATUS_OK:
+            break;
+        case CLI_STATUS_DONE:
+            return 0;
+        case CLI_STATUS_ERROR:
+            return 1;
+    }
     log_close();
 
     log_open(log_backend);
@@ -235,14 +239,14 @@ int main(int argc, char *argv[])
 
     char *pid_file_path = nullptr;
     char *keys_file_path = nullptr;
-    int start_port;
-    int enable_ipv6;
-    int enable_ipv4_fallback;
-    int enable_lan_discovery;
-    int enable_tcp_relay;
+    int start_port = 0;
+    bool enable_ipv6 = false;
+    bool enable_ipv4_fallback = false;
+    bool enable_lan_discovery = false;
+    bool enable_tcp_relay = false;
     uint16_t *tcp_relay_ports = nullptr;
-    int tcp_relay_port_count;
-    int enable_motd;
+    int tcp_relay_port_count = 0;
+    bool enable_motd = false;
     char *motd = nullptr;
 
     if (get_general_config(cfg_file_path, &pid_file_path, &keys_file_path, &start_port, &enable_ipv6, &enable_ipv4_fallback,
@@ -264,7 +268,14 @@ int main(int argc, char *argv[])
     }
 
     if (!run_in_foreground) {
-        daemonize(log_backend, pid_file_path);
+        switch (daemonize(log_backend, pid_file_path)) {
+            case CLI_STATUS_OK:
+                break;
+            case CLI_STATUS_DONE:
+                return 0;
+            case CLI_STATUS_ERROR:
+                return 1;
+        }
     }
 
     free(pid_file_path);
@@ -274,20 +285,22 @@ int main(int argc, char *argv[])
 
     Logger *logger = logger_new();
 
-    if (MIN_LOGGER_LEVEL == LOGGER_LEVEL_TRACE || MIN_LOGGER_LEVEL == LOGGER_LEVEL_DEBUG) {
+    if (MIN_LOGGER_LEVEL <= LOGGER_LEVEL_DEBUG) {
         logger_callback_log(logger, toxcore_logger_callback, nullptr, nullptr);
     }
 
     const uint16_t end_port = start_port + (TOX_PORTRANGE_TO - TOX_PORTRANGE_FROM);
-    const Network *ns = system_network();
-    Networking_Core *net = new_networking_ex(logger, ns, &ip, start_port, end_port, nullptr);
+    const Memory *mem = os_memory();
+    const Random *rng = os_random();
+    const Network *ns = os_network();
+    Networking_Core *net = new_networking_ex(logger, mem, ns, &ip, start_port, end_port, nullptr);
 
     if (net == nullptr) {
         if (enable_ipv6 && enable_ipv4_fallback) {
             log_write(LOG_LEVEL_WARNING, "Couldn't initialize IPv6 networking. Falling back to using IPv4.\n");
-            enable_ipv6 = 0;
+            enable_ipv6 = false;
             ip_init(&ip, enable_ipv6);
-            net = new_networking_ex(logger, ns, &ip, start_port, end_port, nullptr);
+            net = new_networking_ex(logger, mem, ns, &ip, start_port, end_port, nullptr);
 
             if (net == nullptr) {
                 log_write(LOG_LEVEL_ERROR, "Couldn't fallback to IPv4. Exiting.\n");
@@ -307,7 +320,7 @@ int main(int argc, char *argv[])
         }
     }
 
-    Mono_Time *const mono_time = mono_time_new(nullptr, nullptr);
+    Mono_Time *const mono_time = mono_time_new(mem, nullptr, nullptr);
 
     if (mono_time == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't initialize monotonic timer. Exiting.\n");
@@ -321,12 +334,11 @@ int main(int argc, char *argv[])
 
     mono_time_update(mono_time);
 
-    const Random *rng = system_random();
-    DHT *const dht = new_dht(logger, rng, ns, mono_time, net, true, enable_lan_discovery);
+    DHT *const dht = new_dht(logger, mem, rng, ns, mono_time, net, true, enable_lan_discovery);
 
     if (dht == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't initialize Tox DHT instance. Exiting.\n");
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(motd);
@@ -340,7 +352,7 @@ int main(int argc, char *argv[])
     if (forwarding == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't initialize forwarding. Exiting.\n");
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(motd);
@@ -349,13 +361,13 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    Announcements *announce = new_announcements(logger, rng, mono_time, forwarding);
+    Announcements *announce = new_announcements(logger, mem, rng, mono_time, forwarding);
 
     if (announce == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't initialize DHT announcements. Exiting.\n");
         kill_forwarding(forwarding);
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(motd);
@@ -364,14 +376,14 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    Onion *onion = new_onion(logger, mono_time, rng, dht);
+    GC_Announces_List *group_announce = new_gca_list();
 
-    if (!onion) {
-        log_write(LOG_LEVEL_ERROR, "Couldn't initialize Tox Onion. Exiting.\n");
+    if (group_announce == nullptr) {
+        log_write(LOG_LEVEL_ERROR, "Couldn't initialize group announces. Exiting.\n");
         kill_announcements(announce);
         kill_forwarding(forwarding);
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(motd);
@@ -380,15 +392,33 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    Onion_Announce *onion_a = new_onion_announce(logger, rng, mono_time, dht);
+    Onion *onion = new_onion(logger, mem, mono_time, rng, dht);
 
-    if (!onion_a) {
+    if (onion == nullptr) {
+        log_write(LOG_LEVEL_ERROR, "Couldn't initialize Tox Onion. Exiting.\n");
+        kill_gca(group_announce);
+        kill_announcements(announce);
+        kill_forwarding(forwarding);
+        kill_dht(dht);
+        mono_time_free(mem, mono_time);
+        kill_networking(net);
+        logger_kill(logger);
+        free(motd);
+        free(tcp_relay_ports);
+        free(keys_file_path);
+        return 1;
+    }
+
+    Onion_Announce *onion_a = new_onion_announce(logger, mem, rng, mono_time, dht);
+
+    if (onion_a == nullptr) {
         log_write(LOG_LEVEL_ERROR, "Couldn't initialize Tox Onion Announce. Exiting.\n");
+        kill_gca(group_announce);
         kill_onion(onion);
         kill_announcements(announce);
         kill_forwarding(forwarding);
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(motd);
@@ -397,6 +427,8 @@ int main(int argc, char *argv[])
         return 1;
     }
 
+    gca_onion_init(group_announce, onion_a);
+
     if (enable_motd) {
         if (bootstrap_set_callbacks(dht_get_net(dht), DAEMON_VERSION_NUMBER, (uint8_t *)motd, strlen(motd) + 1) == 0) {
             log_write(LOG_LEVEL_INFO, "Set MOTD successfully.\n");
@@ -404,11 +436,12 @@ int main(int argc, char *argv[])
         } else {
             log_write(LOG_LEVEL_ERROR, "Couldn't set MOTD: %s. Exiting.\n", motd);
             kill_onion_announce(onion_a);
+            kill_gca(group_announce);
             kill_onion(onion);
             kill_announcements(announce);
             kill_forwarding(forwarding);
             kill_dht(dht);
-            mono_time_free(mono_time);
+            mono_time_free(mem, mono_time);
             kill_networking(net);
             logger_kill(logger);
             free(motd);
@@ -424,11 +457,12 @@ int main(int argc, char *argv[])
     } else {
         log_write(LOG_LEVEL_ERROR, "Couldn't read/write: %s. Exiting.\n", keys_file_path);
         kill_onion_announce(onion_a);
+        kill_gca(group_announce);
         kill_onion(onion);
         kill_announcements(announce);
         kill_forwarding(forwarding);
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         free(tcp_relay_ports);
@@ -442,18 +476,20 @@ int main(int argc, char *argv[])
         if (tcp_relay_port_count == 0) {
             log_write(LOG_LEVEL_ERROR, "No TCP relay ports read. Exiting.\n");
             kill_onion_announce(onion_a);
+            kill_gca(group_announce);
             kill_announcements(announce);
             kill_forwarding(forwarding);
             kill_onion(onion);
             kill_dht(dht);
-            mono_time_free(mono_time);
+            mono_time_free(mem, mono_time);
             kill_networking(net);
             logger_kill(logger);
             free(tcp_relay_ports);
             return 1;
         }
 
-        tcp_server = new_TCP_server(logger, rng, ns, enable_ipv6, tcp_relay_port_count, tcp_relay_ports,
+        tcp_server = new_tcp_server(logger, mem, rng, ns, enable_ipv6,
+                                    tcp_relay_port_count, tcp_relay_ports,
                                     dht_get_self_secret_key(dht), onion, forwarding);
 
         free(tcp_relay_ports);
@@ -468,7 +504,7 @@ int main(int argc, char *argv[])
 
             assert(rlim_suggested >= rlim_min);
 
-            if (!getrlimit(RLIMIT_NOFILE, &limit)) {
+            if (getrlimit(RLIMIT_NOFILE, &limit) == 0) {
                 if (limit.rlim_cur < limit.rlim_max) {
                     // Some systems have a hard limit of over 1000000 open file descriptors, so let's cap it at something reasonable
                     // so that we don't set it to an unreasonably high number.
@@ -477,7 +513,7 @@ int main(int argc, char *argv[])
                 }
             }
 
-            if (!getrlimit(RLIMIT_NOFILE, &limit) && limit.rlim_cur < rlim_min) {
+            if (getrlimit(RLIMIT_NOFILE, &limit) == 0 && limit.rlim_cur < rlim_min) {
                 log_write(LOG_LEVEL_WARNING,
                           "Current limit on the number of files this process can open (%ju) is rather low for the proper functioning of the TCP server. "
                           "Consider raising the limit to at least %ju or the recommended %ju. "
@@ -487,11 +523,12 @@ int main(int argc, char *argv[])
         } else {
             log_write(LOG_LEVEL_ERROR, "Couldn't initialize Tox TCP server. Exiting.\n");
             kill_onion_announce(onion_a);
+            kill_gca(group_announce);
             kill_onion(onion);
             kill_announcements(announce);
             kill_forwarding(forwarding);
             kill_dht(dht);
-            mono_time_free(mono_time);
+            mono_time_free(mem, mono_time);
             kill_networking(net);
             logger_kill(logger);
             return 1;
@@ -502,13 +539,14 @@ int main(int argc, char *argv[])
         log_write(LOG_LEVEL_INFO, "List of bootstrap nodes read successfully.\n");
     } else {
         log_write(LOG_LEVEL_ERROR, "Couldn't read list of bootstrap nodes in %s. Exiting.\n", cfg_file_path);
-        kill_TCP_server(tcp_server);
+        kill_tcp_server(tcp_server);
         kill_onion_announce(onion_a);
+        kill_gca(group_announce);
         kill_onion(onion);
         kill_announcements(announce);
         kill_forwarding(forwarding);
         kill_dht(dht);
-        mono_time_free(mono_time);
+        mono_time_free(mem, mono_time);
         kill_networking(net);
         logger_kill(logger);
         return 1;
@@ -516,10 +554,10 @@ int main(int argc, char *argv[])
 
     print_public_key(dht_get_self_public_key(dht));
 
-    uint64_t last_LANdiscovery = 0;
+    uint64_t last_lan_discovery = 0;
     const uint16_t net_htons_port = net_htons(start_port);
 
-    int waiting_for_dht_connection = 1;
+    bool waiting_for_dht_connection = true;
 
     Broadcast_Info *broadcast = nullptr;
 
@@ -538,33 +576,33 @@ int main(int argc, char *argv[])
     // Prevent the signal handler from being called again before it returns
     sigfillset(&sa.sa_mask);
 
-    if (sigaction(SIGINT, &sa, nullptr)) {
+    if (sigaction(SIGINT, &sa, nullptr) != 0) {
         log_write(LOG_LEVEL_WARNING, "Couldn't set signal handler for SIGINT. Continuing without the signal handler set.\n");
     }
 
-    if (sigaction(SIGTERM, &sa, nullptr)) {
+    if (sigaction(SIGTERM, &sa, nullptr) != 0) {
         log_write(LOG_LEVEL_WARNING, "Couldn't set signal handler for SIGTERM. Continuing without the signal handler set.\n");
     }
 
-    while (!caught_signal) {
+    while (caught_signal == 0) {
         mono_time_update(mono_time);
 
         do_dht(dht);
 
-        if (enable_lan_discovery && mono_time_is_timeout(mono_time, last_LANdiscovery, LAN_DISCOVERY_INTERVAL)) {
+        if (enable_lan_discovery && mono_time_is_timeout(mono_time, last_lan_discovery, LAN_DISCOVERY_INTERVAL)) {
             lan_discovery_send(dht_get_net(dht), broadcast, dht_get_self_public_key(dht), net_htons_port);
-            last_LANdiscovery = mono_time_get(mono_time);
+            last_lan_discovery = mono_time_get(mono_time);
         }
 
         if (enable_tcp_relay) {
-            do_TCP_server(tcp_server, mono_time);
+            do_tcp_server(tcp_server, mono_time);
         }
 
         networking_poll(dht_get_net(dht), nullptr);
 
         if (waiting_for_dht_connection && dht_isconnected(dht)) {
             log_write(LOG_LEVEL_INFO, "Connected to another bootstrap node successfully.\n");
-            waiting_for_dht_connection = 0;
+            waiting_for_dht_connection = false;
         }
 
         sleep_milliseconds(30);
@@ -580,17 +618,18 @@ int main(int argc, char *argv[])
             break;
 
         default:
-            log_write(LOG_LEVEL_INFO, "Received (%d) signal. Exiting.\n", caught_signal);
+            log_write(LOG_LEVEL_INFO, "Received (%ld) signal. Exiting.\n", (long)caught_signal);
     }
 
     lan_discovery_kill(broadcast);
-    kill_TCP_server(tcp_server);
+    kill_tcp_server(tcp_server);
     kill_onion_announce(onion_a);
+    kill_gca(group_announce);
     kill_onion(onion);
     kill_announcements(announce);
     kill_forwarding(forwarding);
     kill_dht(dht);
-    mono_time_free(mono_time);
+    mono_time_free(mem, mono_time);
     kill_networking(net);
     logger_kill(logger);
 
