@@ -13,6 +13,7 @@
 #include "../toxcore/ccompat.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/mono_time.h"
+#include "../toxcore/network.h"
 
 static struct JitterBuffer *jbuf_new(uint32_t capacity);
 static void jbuf_clear(struct JitterBuffer *q);
@@ -25,6 +26,8 @@ static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, uint32
                                       uint8_t new_ch, uint32_t *old_br, uint32_t *old_sr, uint8_t *old_ch);
 static bool reconfigure_audio_decoder(ACSession *ac, uint32_t sampling_rate, uint8_t channels);
 
+
+
 ACSession *ac_new(Mono_Time *mono_time, const Logger *log, ToxAV *av, uint32_t friend_number,
                   toxav_audio_receive_frame_cb *cb, void *cb_data)
 {
@@ -150,9 +153,9 @@ void ac_iterate(ACSession *ac)
 
             ac->lp_channel_count = opus_packet_get_nb_channels(msg->data + 4);
 
-            /* NOTE: even though OPUS supports decoding mono frames with stereo decoder and vice versa,
-             * it didn't work quite well.
-             */
+            /** NOTE: even though OPUS supports decoding mono frames with stereo decoder and vice versa,
+              * it didn't work quite well.
+              */
             if (!reconfigure_audio_decoder(ac, ac->lp_sampling_rate, ac->lp_channel_count)) {
                 LOGGER_WARNING(ac->log, "Failed to reconfigure decoder!");
                 free(msg);
@@ -273,6 +276,7 @@ static struct JitterBuffer *jbuf_new(uint32_t capacity)
     q->capacity = capacity;
     return q;
 }
+
 static void jbuf_clear(struct JitterBuffer *q)
 {
     while (q->bottom != q->top) {
@@ -281,6 +285,7 @@ static void jbuf_clear(struct JitterBuffer *q)
         ++q->bottom;
     }
 }
+
 static void jbuf_free(struct JitterBuffer *q)
 {
     if (q == nullptr) {
@@ -291,6 +296,11 @@ static void jbuf_free(struct JitterBuffer *q)
     free(q->queue);
     free(q);
 }
+
+/*
+ * if -1 is returned the RTPMessage m needs to be free'd by the caller
+ * if  0 is returned the RTPMessage m is stored in the ringbuffer and must NOT be freed by the caller
+ */
 static int jbuf_write(const Logger *log, struct JitterBuffer *q, struct RTPMessage *m)
 {
     const uint16_t sequnum = m->header.sequnum;
@@ -319,6 +329,7 @@ static int jbuf_write(const Logger *log, struct JitterBuffer *q, struct RTPMessa
 
     return 0;
 }
+
 static struct RTPMessage *jbuf_read(struct JitterBuffer *q, int32_t *success)
 {
     if (q->top == q->bottom) {
@@ -377,6 +388,21 @@ static OpusEncoder *create_audio_encoder(const Logger *log, uint32_t bit_rate, u
         goto FAILURE;
     }
 
+    /*
+     * The libopus library defaults to VBR, which is unsafe in any VoIP environment
+     * (see for example doi:10.1109/SP.2011.34). Switching to CBR very slightly
+     * decreases audio quality at lower bitrates.
+     *
+     * Parameters:
+     *  `[in]`    `x`   `opus_int32`: Whether to use VBR mode, 1 (VBR) is default
+     */
+    status = opus_encoder_ctl(rc, OPUS_SET_VBR(0));
+
+    if (status != OPUS_OK) {
+        LOGGER_ERROR(log, "Error while setting encoder ctl: %s", opus_strerror(status));
+        goto FAILURE;
+    }
+
     /*
      * Configures the encoder's use of inband forward error correction.
      * Note:
