@@ -67,7 +67,7 @@ ACSession *ac_new(Mono_Time *mono_time, const Logger *log, ToxAV *av, uint32_t f
                   toxav_audio_receive_frame_cb *cb, void *cb_data);
 void ac_kill(ACSession *ac);
 void ac_iterate(ACSession *ac);
-int ac_queue_message(Mono_Time *mono_time, void *acp, struct RTPMessage *msg);
+int ac_queue_message(Mono_Time *mono_time, void *cs, struct RTPMessage *msg);
 int ac_reconfigure_encoder(ACSession *ac, uint32_t bit_rate, uint32_t sampling_rate, uint8_t channels);
 
-#endif // C_TOXCORE_TOXAV_AUDIO_H
+#endif /* C_TOXCORE_TOXAV_AUDIO_H */
