@@ -9,6 +9,7 @@
 #ifndef C_TOXCORE_TOXCORE_LAN_DISCOVERY_H
 #define C_TOXCORE_TOXCORE_LAN_DISCOVERY_H
 
+#include "attributes.h"
 #include "network.h"
 
 /**
@@ -24,7 +25,8 @@ typedef struct Broadcast_Info Broadcast_Info;
  * @return true on success, false on failure.
  */
 non_null()
-bool lan_discovery_send(const Networking_Core *net, const Broadcast_Info *broadcast, const uint8_t *dht_pk, uint16_t port);
+bool lan_discovery_send(const Networking_Core *net, const Broadcast_Info *broadcast, const uint8_t *dht_pk,
+                        uint16_t port);
 
 /**
  * Discovers broadcast devices and IP addresses.
@@ -52,4 +54,4 @@ bool ip_is_local(const IP *ip);
 non_null()
 bool ip_is_lan(const IP *ip);
 
-#endif // C_TOXCORE_TOXCORE_LAN_DISCOVERY_H
+#endif /* C_TOXCORE_TOXCORE_LAN_DISCOVERY_H */
