@@ -10,13 +10,25 @@
 #ifndef C_TOXCORE_TOXCORE_MESSENGER_H
 #define C_TOXCORE_TOXCORE_MESSENGER_H
 
+#include "DHT.h"
+#include "TCP_client.h"
 #include "TCP_server.h"
 #include "announce.h"
+#include "attributes.h"
+#include "crypto_core.h"
 #include "forwarding.h"
 #include "friend_connection.h"
 #include "friend_requests.h"
+#include "group_announce.h"
+#include "group_common.h"
 #include "logger.h"
+#include "mem.h"
+#include "mono_time.h"
 #include "net_crypto.h"
+#include "network.h"
+#include "onion.h"
+#include "onion_announce.h"
+#include "onion_client.h"
 #include "state.h"
 
 #define MAX_NAME_LENGTH 128
@@ -27,7 +39,6 @@
 /* This cannot be bigger than 256 */
 #define MAX_CONCURRENT_FILE_PIPES 256
 
-
 #define FRIEND_ADDRESS_SIZE (CRYPTO_PUBLIC_KEY_SIZE + sizeof(uint32_t) + sizeof(uint16_t))
 
 typedef enum Message_Type {
@@ -35,7 +46,11 @@ typedef enum Message_Type {
     MESSAGE_ACTION,
 } Message_Type;
 
+// TODO(Jfreegman, Iphy): Remove this before merge
+#ifndef MESSENGER_DEFINED
+#define MESSENGER_DEFINED
 typedef struct Messenger Messenger;
+#endif /* MESSENGER_DEFINED */
 
 // Returns the size of the data
 typedef uint32_t m_state_size_cb(const Messenger *m);
@@ -63,6 +78,7 @@ typedef struct Messenger_Options {
     bool hole_punching_enabled;
     bool local_discovery_enabled;
     bool dht_announcements_enabled;
+    bool groups_persistence_enabled;
 
     logger_cb *log_callback;
     void *log_context;
@@ -72,7 +88,6 @@ typedef struct Messenger_Options {
     uint8_t state_plugins_length;
 } Messenger_Options;
 
-
 struct Receipts {
     uint32_t packet_num;
     uint32_t msg_id;
@@ -102,7 +117,6 @@ typedef enum Friend_Add_Error {
     FAERR_NOMEM = -8,
 } Friend_Add_Error;
 
-
 /** Default start timeout in seconds between friend requests. */
 #define FRIENDREQUEST_TIMEOUT 5
 
@@ -160,7 +174,6 @@ typedef enum Filekind {
     FILEKIND_AVATAR,
 } Filekind;
 
-
 typedef void m_self_connection_status_cb(Messenger *m, Onion_Connection_Status connection_status, void *user_data);
 typedef void m_friend_status_cb(Messenger *m, uint32_t friend_number, unsigned int status, void *user_data);
 typedef void m_friend_connection_status_cb(Messenger *m, uint32_t friend_number, unsigned int connection_status,
@@ -187,18 +200,10 @@ typedef void m_friend_lossy_packet_cb(Messenger *m, uint32_t friend_number, uint
                                       size_t length, void *user_data);
 typedef void m_friend_lossless_packet_cb(Messenger *m, uint32_t friend_number, uint8_t packet_id, const uint8_t *data,
         size_t length, void *user_data);
-typedef void m_friend_connectionstatuschange_internal_cb(Messenger *m, uint32_t friend_number,
-        uint8_t connection_status, void *user_data);
 typedef void m_conference_invite_cb(Messenger *m, uint32_t friend_number, const uint8_t *cookie, uint16_t length,
                                     void *user_data);
-typedef void m_msi_packet_cb(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length,
-                             void *user_data);
-typedef int m_lossy_rtp_packet_cb(Messenger *m, uint32_t friendnumber, const uint8_t *data, uint16_t len, void *object);
-
-typedef struct RTP_Packet_Handler {
-    m_lossy_rtp_packet_cb *function;
-    void *object;
-} RTP_Packet_Handler;
+typedef void m_group_invite_cb(const Messenger *m, uint32_t friend_number, const uint8_t *invite_data, size_t length,
+                               const uint8_t *group_name, size_t group_name_length, void *user_data);
 
 typedef struct Friend {
     uint8_t real_pk[CRYPTO_PUBLIC_KEY_SIZE];
@@ -228,8 +233,6 @@ typedef struct Friend {
     uint32_t num_sending_files;
     struct File_Transfers file_receiving[MAX_CONCURRENT_FILE_PIPES];
 
-    RTP_Packet_Handler lossy_rtp_packethandlers[PACKET_ID_RANGE_LOSSY_AV_SIZE];
-
     struct Receipts *receipts_start;
     struct Receipts *receipts_end;
 } Friend;
@@ -237,6 +240,7 @@ typedef struct Friend {
 struct Messenger {
     Logger *log;
     Mono_Time *mono_time;
+    const Memory *mem;
     const Random *rng;
     const Network *ns;
 
@@ -269,6 +273,9 @@ struct Messenger {
     uint64_t lastdump;
     uint8_t is_receiving_file;
 
+    GC_Session *group_handler;
+    GC_Announces_List *group_announce;
+
     bool has_added_relays; // If the first connection has occurred in do_messenger
 
     uint16_t num_loaded_relays;
@@ -282,20 +289,17 @@ struct Messenger {
     m_friend_typing_cb *friend_typingchange;
     m_friend_read_receipt_cb *read_receipt;
     m_friend_connection_status_cb *friend_connectionstatuschange;
-    m_friend_connectionstatuschange_internal_cb *friend_connectionstatuschange_internal;
-    void *friend_connectionstatuschange_internal_userdata;
 
-    struct Group_Chats *conferences_object; /* Set by new_groupchats()*/
+    struct Group_Chats *conferences_object;
     m_conference_invite_cb *conference_invite;
 
+    m_group_invite_cb *group_invite;
+
     m_file_recv_cb *file_sendrequest;
     m_file_recv_control_cb *file_filecontrol;
     m_file_recv_chunk_cb *file_filedata;
     m_file_chunk_request_cb *file_reqchunk;
 
-    m_msi_packet_cb *msi_packet;
-    void *msi_packet_userdata;
-
     m_friend_lossy_packet_cb *lossy_packethandler;
     m_friend_lossless_packet_cb *lossless_packethandler;
 
@@ -305,6 +309,14 @@ struct Messenger {
     Messenger_Options options;
 };
 
+/**
+ * Determines if the friendnumber passed is valid in the Messenger object.
+ *
+ * @param friendnumber The index in the friend list.
+ */
+non_null()
+bool friend_is_valid(const Messenger *m, int32_t friendnumber);
+
 /**
  * Format: `[real_pk (32 bytes)][nospam number (4 bytes)][checksum (2 bytes)]`
  *
@@ -337,7 +349,6 @@ void getaddress(const Messenger *m, uint8_t *address);
 non_null()
 int32_t m_addfriend(Messenger *m, const uint8_t *address, const uint8_t *data, uint16_t length);
 
-
 /** @brief Add a friend without sending a friendrequest.
  * @return the friend number if success.
  * @retval -3 if user's own key.
@@ -348,6 +359,19 @@ int32_t m_addfriend(Messenger *m, const uint8_t *address, const uint8_t *data, u
 non_null()
 int32_t m_addfriend_norequest(Messenger *m, const uint8_t *real_pk);
 
+/** @brief Initializes the friend connection and onion connection for a groupchat.
+ *
+ * @retval true on success.
+ */
+non_null()
+bool m_create_group_connection(Messenger *m, GC_Chat *chat);
+
+/*
+ * Kills the friend connection for a groupchat.
+ */
+non_null()
+void m_kill_group_connection(Messenger *m, const GC_Chat *chat);
+
 /** @return the friend number associated to that public key.
  * @retval -1 if no such friend.
  */
@@ -414,7 +438,6 @@ non_null(1, 4) nullable(6)
 int m_send_message_generic(Messenger *m, int32_t friendnumber, uint8_t type, const uint8_t *message, uint32_t length,
                            uint32_t *message_id);
 
-
 /** @brief Set the name and name_length of a friend.
  *
  * name must be a string of maximum MAX_NAME_LENGTH length.
@@ -505,7 +528,6 @@ non_null() int m_copy_self_statusmessage(const Messenger *m, uint8_t *buf);
 non_null() uint8_t m_get_userstatus(const Messenger *m, int32_t friendnumber);
 non_null() uint8_t m_get_self_userstatus(const Messenger *m);
 
-
 /** @brief returns timestamp of last time friendnumber was seen online or 0 if never seen.
  * if friendnumber is invalid this function will return UINT64_MAX.
  */
@@ -575,11 +597,6 @@ non_null() void m_callback_read_receipt(Messenger *m, m_friend_read_receipt_cb *
  */
 non_null() void m_callback_connectionstatus(Messenger *m, m_friend_connection_status_cb *function);
 
-/** Same as previous but for internal A/V core usage only */
-non_null() void m_callback_connectionstatus_internal_av(
-    Messenger *m, m_friend_connectionstatuschange_internal_cb *function, void *userdata);
-
-
 /** @brief Set the callback for typing changes. */
 non_null() void m_callback_core_connection(Messenger *m, m_self_connection_status_cb *function);
 
@@ -589,6 +606,11 @@ non_null() void m_callback_core_connection(Messenger *m, m_self_connection_statu
 non_null(1) nullable(2)
 void m_callback_conference_invite(Messenger *m, m_conference_invite_cb *function);
 
+/* Set the callback for group invites.
+ */
+non_null(1) nullable(2)
+void m_callback_group_invite(Messenger *m, m_group_invite_cb *function);
+
 /** @brief Send a conference invite packet.
  *
  * return true on success
@@ -597,13 +619,21 @@ void m_callback_conference_invite(Messenger *m, m_conference_invite_cb *function
 non_null()
 bool send_conference_invite_packet(const Messenger *m, int32_t friendnumber, const uint8_t *data, uint16_t length);
 
-/*** FILE SENDING */
+/* Send a group invite packet.
+ *
+ *  WARNING: Return-value semantics are different than for
+ *  send_conference_invite_packet().
+ *
+ *  return true on success
+ */
+non_null()
+bool send_group_invite_packet(const Messenger *m, uint32_t friendnumber, const uint8_t *packet, uint16_t length);
 
+/*** FILE SENDING */
 
 /** @brief Set the callback for file send requests. */
 non_null() void callback_file_sendrequest(Messenger *m, m_file_recv_cb *function);
 
-
 /** @brief Set the callback for file control requests. */
 non_null() void callback_file_control(Messenger *m, m_file_recv_control_cb *function);
 
@@ -613,7 +643,6 @@ non_null() void callback_file_data(Messenger *m, m_file_recv_chunk_cb *function)
 /** @brief Set the callback for file request chunk. */
 non_null() void callback_file_reqchunk(Messenger *m, m_file_chunk_request_cb *function);
 
-
 /** @brief Copy the file transfer file id to file_id
  *
  * @retval 0 on success.
@@ -678,30 +707,8 @@ int file_seek(const Messenger *m, int32_t friendnumber, uint32_t filenumber, uin
  * @retval -7 if wrong position.
  */
 non_null(1) nullable(5)
-int send_file_data(const Messenger *m, int32_t friendnumber, uint32_t filenumber, uint64_t position, const uint8_t *data, uint16_t length);
-
-/*** A/V related */
-
-/** @brief Set the callback for msi packets. */
-non_null(1) nullable(2, 3)
-void m_callback_msi_packet(Messenger *m, m_msi_packet_cb *function, void *userdata);
-
-/** @brief Send an msi packet.
- *
- * @retval true on success
- * @retval false on failure
- */
-non_null()
-bool m_msi_packet(const Messenger *m, int32_t friendnumber, const uint8_t *data, uint16_t length);
-
-/** @brief Set handlers for lossy rtp packets.
- *
- * @retval -1 on failure.
- * @retval 0 on success.
- */
-non_null(1) nullable(4, 5)
-int m_callback_rtp_packet(Messenger *m, int32_t friendnumber, uint8_t byte,
-                          m_lossy_rtp_packet_cb *function, void *object);
+int send_file_data(const Messenger *m, int32_t friendnumber, uint32_t filenumber, uint64_t position,
+                   const uint8_t *data, uint16_t length);
 
 /*** CUSTOM PACKETS */
 
@@ -709,13 +716,6 @@ int m_callback_rtp_packet(Messenger *m, int32_t friendnumber, uint8_t byte,
 non_null() void custom_lossy_packet_registerhandler(Messenger *m, m_friend_lossy_packet_cb *lossy_packethandler);
 
 /** @brief High level function to send custom lossy packets.
- *
- * TODO(oxij): this name is confusing, because this function sends both av and custom lossy packets.
- * Meanwhile, m_handle_lossy_packet routes custom packets to custom_lossy_packet_registerhandler
- * as you would expect from its name.
- *
- * I.e. custom_lossy_packet_registerhandler's "custom lossy packet" and this "custom lossy packet"
- * are not the same set of packets.
  *
  * @retval -1 if friend invalid.
  * @retval -2 if length wrong.
@@ -727,7 +727,6 @@ non_null() void custom_lossy_packet_registerhandler(Messenger *m, m_friend_lossy
 non_null()
 int m_send_custom_lossy_packet(const Messenger *m, int32_t friendnumber, const uint8_t *data, uint32_t length);
 
-
 /** @brief Set handlers for custom lossless packets. */
 non_null()
 void custom_lossless_packet_registerhandler(Messenger *m, m_friend_lossless_packet_cb *lossless_packethandler);
@@ -761,7 +760,8 @@ typedef enum Messenger_Error {
  * if error is not NULL it will be set to one of the values in the enum above.
  */
 non_null()
-Messenger *new_messenger(Mono_Time *mono_time, const Random *rng, const Network *ns, Messenger_Options *options, Messenger_Error *error);
+Messenger *new_messenger(Mono_Time *mono_time, const Memory *mem, const Random *rng, const Network *ns,
+                         Messenger_Options *options, Messenger_Error *error);
 
 /** @brief Run this before closing shop.
  *
@@ -836,4 +836,4 @@ uint32_t copy_friendlist(const Messenger *m, uint32_t *out_list, uint32_t list_s
 non_null()
 bool m_is_receiving_file(Messenger *m);
 
-#endif
+#endif /* C_TOXCORE_TOXCORE_MESSENGER_H */
