@@ -1,44 +1,19 @@
 /* SPDX-License-Identifier: GPL-3.0-or-later
- * Copyright © 2016-2018 The TokTok team.
+ * Copyright © 2016-2024 The TokTok team.
  * Copyright © 2013 Tox project.
  */
 
-/**
- * Functions for the core crypto.
- *
- * NOTE: This code has to be perfect. We don't mess around with encryption.
- */
 #include "crypto_core.h"
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef VANILLA_NACL
-// We use libsodium by default.
 #include <sodium.h>
-#else
-#include <crypto_auth.h>
-#include <crypto_box.h>
-#include <crypto_hash_sha256.h>
-#include <crypto_hash_sha512.h>
-#include <crypto_scalarmult_curve25519.h>
-#include <crypto_verify_16.h>
-#include <crypto_verify_32.h>
-#include <randombytes.h>
-#endif
 
+#include "attributes.h"
 #include "ccompat.h"
-
-#ifndef crypto_box_MACBYTES
-#define crypto_box_MACBYTES (crypto_box_ZEROBYTES - crypto_box_BOXZEROBYTES)
-#endif
-
-#ifndef VANILLA_NACL
-// Need dht because of ENC_SECRET_KEY_SIZE and ENC_PUBLIC_KEY_SIZE
-#define ENC_PUBLIC_KEY_SIZE CRYPTO_PUBLIC_KEY_SIZE
-#define ENC_SECRET_KEY_SIZE CRYPTO_SECRET_KEY_SIZE
-#endif
+#include "util.h"
 
 static_assert(CRYPTO_PUBLIC_KEY_SIZE == crypto_box_PUBLICKEYBYTES,
               "CRYPTO_PUBLIC_KEY_SIZE should be equal to crypto_box_PUBLICKEYBYTES");
@@ -63,56 +38,53 @@ static_assert(CRYPTO_SHA512_SIZE == crypto_hash_sha512_BYTES,
 static_assert(CRYPTO_PUBLIC_KEY_SIZE == 32,
               "CRYPTO_PUBLIC_KEY_SIZE is required to be 32 bytes for pk_equal to work");
 
-#ifndef VANILLA_NACL
 static_assert(CRYPTO_SIGNATURE_SIZE == crypto_sign_BYTES,
               "CRYPTO_SIGNATURE_SIZE should be equal to crypto_sign_BYTES");
 static_assert(CRYPTO_SIGN_PUBLIC_KEY_SIZE == crypto_sign_PUBLICKEYBYTES,
               "CRYPTO_SIGN_PUBLIC_KEY_SIZE should be equal to crypto_sign_PUBLICKEYBYTES");
 static_assert(CRYPTO_SIGN_SECRET_KEY_SIZE == crypto_sign_SECRETKEYBYTES,
               "CRYPTO_SIGN_SECRET_KEY_SIZE should be equal to crypto_sign_SECRETKEYBYTES");
-#endif /* VANILLA_NACL */
 
-bool create_extended_keypair(uint8_t *pk, uint8_t *sk)
+bool create_extended_keypair(Extended_Public_Key *pk, Extended_Secret_Key *sk, const Random *rng)
 {
-#ifdef VANILLA_NACL
-    return false;
-#else
     /* create signature key pair */
-    crypto_sign_keypair(pk + ENC_PUBLIC_KEY_SIZE, sk + ENC_SECRET_KEY_SIZE);
+    uint8_t seed[crypto_sign_SEEDBYTES];
+    random_bytes(rng, seed, crypto_sign_SEEDBYTES);
+    crypto_sign_seed_keypair(pk->sig, sk->sig, seed);
+    crypto_memzero(seed, crypto_sign_SEEDBYTES);
 
     /* convert public signature key to public encryption key */
-    const int res1 = crypto_sign_ed25519_pk_to_curve25519(pk, pk + ENC_PUBLIC_KEY_SIZE);
+    const int res1 = crypto_sign_ed25519_pk_to_curve25519(pk->enc, pk->sig);
 
     /* convert secret signature key to secret encryption key */
-    const int res2 = crypto_sign_ed25519_sk_to_curve25519(sk, sk + ENC_SECRET_KEY_SIZE);
+    const int res2 = crypto_sign_ed25519_sk_to_curve25519(sk->enc, sk->sig);
 
     return res1 == 0 && res2 == 0;
-#endif
 }
 
-const uint8_t *get_enc_key(const uint8_t *key)
+const uint8_t *get_enc_key(const Extended_Public_Key *key)
 {
-    return key;
+    return key->enc;
 }
 
-const uint8_t *get_sig_pk(const uint8_t *key)
+const uint8_t *get_sig_pk(const Extended_Public_Key *key)
 {
-    return key + ENC_PUBLIC_KEY_SIZE;
+    return key->sig;
 }
 
-void set_sig_pk(uint8_t *key, const uint8_t *sig_pk)
+void set_sig_pk(Extended_Public_Key *key, const uint8_t *sig_pk)
 {
-    memcpy(key + ENC_PUBLIC_KEY_SIZE, sig_pk, SIG_PUBLIC_KEY_SIZE);
+    memcpy(key->sig, sig_pk, SIG_PUBLIC_KEY_SIZE);
 }
 
-const uint8_t *get_sig_sk(const uint8_t *key)
+const uint8_t *get_sig_sk(const Extended_Secret_Key *key)
 {
-    return key + ENC_SECRET_KEY_SIZE;
+    return key->sig;
 }
 
-const uint8_t *get_chat_id(const uint8_t *key)
+const uint8_t *get_chat_id(const Extended_Public_Key *key)
 {
-    return key + ENC_PUBLIC_KEY_SIZE;
+    return key->sig;
 }
 
 #if !defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
@@ -137,82 +109,70 @@ static void crypto_free(uint8_t *ptr, size_t bytes)
 
     free(ptr);
 }
-#endif  // !defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
+#endif /* !defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) */
 
 void crypto_memzero(void *data, size_t length)
 {
-#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) || defined(VANILLA_NACL)
-    memset(data, 0, length);
+#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
+    memzero((uint8_t *)data, length);
 #else
     sodium_memzero(data, length);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
 bool crypto_memlock(void *data, size_t length)
 {
-#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) || defined(VANILLA_NACL)
+#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
     return false;
 #else
 
-    if (sodium_mlock(data, length) != 0) {
-        return false;
-    }
-
-    return true;
-#endif
+    return sodium_mlock(data, length) == 0;
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
 bool crypto_memunlock(void *data, size_t length)
 {
-#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION) || defined(VANILLA_NACL)
+#if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
     return false;
 #else
 
-    if (sodium_munlock(data, length) != 0) {
-        return false;
-    }
-
-    return true;
-#endif
+    return sodium_munlock(data, length) == 0;
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-bool pk_equal(const uint8_t *pk1, const uint8_t *pk2)
+bool pk_equal(const uint8_t pk1[CRYPTO_PUBLIC_KEY_SIZE], const uint8_t pk2[CRYPTO_PUBLIC_KEY_SIZE])
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     // Hope that this is better for the fuzzer
     return memcmp(pk1, pk2, CRYPTO_PUBLIC_KEY_SIZE) == 0;
 #else
     return crypto_verify_32(pk1, pk2) == 0;
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-void pk_copy(uint8_t *dest, const uint8_t *src)
+void pk_copy(uint8_t dest[CRYPTO_PUBLIC_KEY_SIZE], const uint8_t src[CRYPTO_PUBLIC_KEY_SIZE])
 {
     memcpy(dest, src, CRYPTO_PUBLIC_KEY_SIZE);
 }
 
-bool crypto_sha512_eq(const uint8_t *cksum1, const uint8_t *cksum2)
+bool crypto_sha512_eq(const uint8_t cksum1[CRYPTO_SHA512_SIZE], const uint8_t cksum2[CRYPTO_SHA512_SIZE])
 {
 #if defined(FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION)
     // Hope that this is better for the fuzzer
     return memcmp(cksum1, cksum2, CRYPTO_SHA512_SIZE) == 0;
-#elif defined(VANILLA_NACL)
-    const int lo = crypto_verify_32(cksum1, cksum2) == 0 ? 1 : 0;
-    const int hi = crypto_verify_32(cksum1 + 8, cksum2 + 8) == 0 ? 1 : 0;
-    return (lo & hi) == 1;
 #else
     return crypto_verify_64(cksum1, cksum2) == 0;
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-bool crypto_sha256_eq(const uint8_t *cksum1, const uint8_t *cksum2)
+bool crypto_sha256_eq(const uint8_t cksum1[CRYPTO_SHA256_SIZE], const uint8_t cksum2[CRYPTO_SHA256_SIZE])
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     // Hope that this is better for the fuzzer
     return memcmp(cksum1, cksum2, CRYPTO_SHA256_SIZE) == 0;
 #else
     return crypto_verify_32(cksum1, cksum2) == 0;
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
 uint8_t random_u08(const Random *rng)
@@ -248,44 +208,40 @@ uint32_t random_range_u32(const Random *rng, uint32_t upper_bound)
     return rng->funcs->random_uniform(rng->obj, upper_bound);
 }
 
-bool crypto_signature_create(uint8_t *signature, const uint8_t *message, uint64_t message_length,
-                             const uint8_t *secret_key)
+bool crypto_signature_create(uint8_t signature[CRYPTO_SIGNATURE_SIZE],
+                             const uint8_t *message, uint64_t message_length,
+                             const uint8_t secret_key[SIG_SECRET_KEY_SIZE])
 {
-#ifdef VANILLA_NACL
-    return false;
-#else
     return crypto_sign_detached(signature, nullptr, message, message_length, secret_key) == 0;
-#endif // VANILLA_NACL
 }
 
-bool crypto_signature_verify(const uint8_t *signature, const uint8_t *message, uint64_t message_length,
-                             const uint8_t *public_key)
+bool crypto_signature_verify(const uint8_t signature[CRYPTO_SIGNATURE_SIZE],
+                             const uint8_t *message, uint64_t message_length,
+                             const uint8_t public_key[SIG_PUBLIC_KEY_SIZE])
 {
-#ifdef VANILLA_NACL
-    return false;
-#else
     return crypto_sign_verify_detached(signature, message, message_length, public_key) == 0;
-#endif
 }
 
-bool public_key_valid(const uint8_t *public_key)
+bool public_key_valid(const uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE])
 {
     /* Last bit of key is always zero. */
     return public_key[31] < 128;
 }
 
-int32_t encrypt_precompute(const uint8_t *public_key, const uint8_t *secret_key,
-                           uint8_t *shared_key)
+int32_t encrypt_precompute(const uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE],
+                           const uint8_t secret_key[CRYPTO_SECRET_KEY_SIZE],
+                           uint8_t shared_key[CRYPTO_SHARED_KEY_SIZE])
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     memcpy(shared_key, public_key, CRYPTO_SHARED_KEY_SIZE);
     return 0;
 #else
     return crypto_box_beforenm(shared_key, public_key, secret_key);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-int32_t encrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
+int32_t encrypt_data_symmetric(const uint8_t shared_key[CRYPTO_SHARED_KEY_SIZE],
+                               const uint8_t nonce[CRYPTO_NONCE_SIZE],
                                const uint8_t *plain, size_t length, uint8_t *encrypted)
 {
     if (length == 0 || shared_key == nullptr || nonce == nullptr || plain == nullptr || encrypted == nullptr) {
@@ -296,7 +252,7 @@ int32_t encrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
     // Don't encrypt anything.
     memcpy(encrypted, plain, length);
     // Zero MAC to avoid uninitialized memory reads.
-    memset(encrypted + length, 0, crypto_box_MACBYTES);
+    memzero(encrypted + length, crypto_box_MACBYTES);
 #else
 
     const size_t size_temp_plain = length + crypto_box_ZEROBYTES;
@@ -314,9 +270,9 @@ int32_t encrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
     // crypto_box_afternm requires the entire range of the output array be
     // initialised with something. It doesn't matter what it's initialised with,
     // so we'll pick 0x00.
-    memset(temp_encrypted, 0, size_temp_encrypted);
+    memzero(temp_encrypted, size_temp_encrypted);
 
-    memset(temp_plain, 0, crypto_box_ZEROBYTES);
+    memzero(temp_plain, crypto_box_ZEROBYTES);
     // Pad the message with 32 0 bytes.
     memcpy(temp_plain + crypto_box_ZEROBYTES, plain, length);
 
@@ -332,12 +288,13 @@ int32_t encrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
 
     crypto_free(temp_plain, size_temp_plain);
     crypto_free(temp_encrypted, size_temp_encrypted);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
     assert(length < INT32_MAX - crypto_box_MACBYTES);
     return (int32_t)(length + crypto_box_MACBYTES);
 }
 
-int32_t decrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
+int32_t decrypt_data_symmetric(const uint8_t shared_key[CRYPTO_SHARED_KEY_SIZE],
+                               const uint8_t nonce[CRYPTO_NONCE_SIZE],
                                const uint8_t *encrypted, size_t length, uint8_t *plain)
 {
     if (length <= crypto_box_BOXZEROBYTES || shared_key == nullptr || nonce == nullptr || encrypted == nullptr
@@ -365,9 +322,9 @@ int32_t decrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
     // crypto_box_open_afternm requires the entire range of the output array be
     // initialised with something. It doesn't matter what it's initialised with,
     // so we'll pick 0x00.
-    memset(temp_plain, 0, size_temp_plain);
+    memzero(temp_plain, size_temp_plain);
 
-    memset(temp_encrypted, 0, crypto_box_BOXZEROBYTES);
+    memzero(temp_encrypted, crypto_box_BOXZEROBYTES);
     // Pad the message with 16 0 bytes.
     memcpy(temp_encrypted + crypto_box_BOXZEROBYTES, encrypted, length);
 
@@ -382,13 +339,15 @@ int32_t decrypt_data_symmetric(const uint8_t *shared_key, const uint8_t *nonce,
 
     crypto_free(temp_plain, size_temp_plain);
     crypto_free(temp_encrypted, size_temp_encrypted);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
     assert(length > crypto_box_MACBYTES);
     assert(length < INT32_MAX);
     return (int32_t)(length - crypto_box_MACBYTES);
 }
 
-int32_t encrypt_data(const uint8_t *public_key, const uint8_t *secret_key, const uint8_t *nonce,
+int32_t encrypt_data(const uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE],
+                     const uint8_t secret_key[CRYPTO_SECRET_KEY_SIZE],
+                     const uint8_t nonce[CRYPTO_NONCE_SIZE],
                      const uint8_t *plain, size_t length, uint8_t *encrypted)
 {
     if (public_key == nullptr || secret_key == nullptr) {
@@ -402,7 +361,9 @@ int32_t encrypt_data(const uint8_t *public_key, const uint8_t *secret_key, const
     return ret;
 }
 
-int32_t decrypt_data(const uint8_t *public_key, const uint8_t *secret_key, const uint8_t *nonce,
+int32_t decrypt_data(const uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE],
+                     const uint8_t secret_key[CRYPTO_SECRET_KEY_SIZE],
+                     const uint8_t nonce[CRYPTO_NONCE_SIZE],
                      const uint8_t *encrypted, size_t length, uint8_t *plain)
 {
     if (public_key == nullptr || secret_key == nullptr) {
@@ -416,7 +377,7 @@ int32_t decrypt_data(const uint8_t *public_key, const uint8_t *secret_key, const
     return ret;
 }
 
-void increment_nonce(uint8_t *nonce)
+void increment_nonce(uint8_t nonce[CRYPTO_NONCE_SIZE])
 {
     /* TODO(irungentoo): use `increment_nonce_number(nonce, 1)` or
      * sodium_increment (change to little endian).
@@ -435,7 +396,7 @@ void increment_nonce(uint8_t *nonce)
     }
 }
 
-void increment_nonce_number(uint8_t *nonce, uint32_t increment)
+void increment_nonce_number(uint8_t nonce[CRYPTO_NONCE_SIZE], uint32_t increment)
 {
     /* NOTE don't use breaks inside this loop
      * In particular, make sure, as far as possible,
@@ -457,68 +418,76 @@ void increment_nonce_number(uint8_t *nonce, uint32_t increment)
     }
 }
 
-void random_nonce(const Random *rng, uint8_t *nonce)
+void random_nonce(const Random *rng, uint8_t nonce[CRYPTO_NONCE_SIZE])
 {
     random_bytes(rng, nonce, crypto_box_NONCEBYTES);
 }
 
-void new_symmetric_key(const Random *rng, uint8_t *key)
+void new_symmetric_key(const Random *rng, uint8_t key[CRYPTO_SYMMETRIC_KEY_SIZE])
 {
     random_bytes(rng, key, CRYPTO_SYMMETRIC_KEY_SIZE);
 }
 
-int32_t crypto_new_keypair(const Random *rng, uint8_t *public_key, uint8_t *secret_key)
+int32_t crypto_new_keypair(const Random *rng,
+                           uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE],
+                           uint8_t secret_key[CRYPTO_SECRET_KEY_SIZE])
 {
-#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     random_bytes(rng, secret_key, CRYPTO_SECRET_KEY_SIZE);
-    memset(public_key, 0, CRYPTO_PUBLIC_KEY_SIZE);  // Make MSAN happy
-    crypto_scalarmult_curve25519_base(public_key, secret_key);
+    memzero(public_key, CRYPTO_PUBLIC_KEY_SIZE);  // Make MSAN happy
+    crypto_derive_public_key(public_key, secret_key);
     return 0;
-#else
-    return crypto_box_keypair(public_key, secret_key);
-#endif
 }
 
-void crypto_derive_public_key(uint8_t *public_key, const uint8_t *secret_key)
+void crypto_derive_public_key(uint8_t public_key[CRYPTO_PUBLIC_KEY_SIZE],
+                              const uint8_t secret_key[CRYPTO_SECRET_KEY_SIZE])
 {
     crypto_scalarmult_curve25519_base(public_key, secret_key);
 }
 
-void new_hmac_key(const Random *rng, uint8_t *key)
+void new_hmac_key(const Random *rng, uint8_t key[CRYPTO_HMAC_KEY_SIZE])
 {
     random_bytes(rng, key, CRYPTO_HMAC_KEY_SIZE);
 }
 
-void crypto_hmac(uint8_t auth[CRYPTO_HMAC_SIZE], const uint8_t key[CRYPTO_HMAC_KEY_SIZE], const uint8_t *data,
-                 size_t length)
+void crypto_hmac(uint8_t auth[CRYPTO_HMAC_SIZE], const uint8_t key[CRYPTO_HMAC_KEY_SIZE],
+                 const uint8_t *data, size_t length)
 {
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+    memcpy(auth, key, 16);
+    memcpy(auth + 16, data, length < 16 ? length : 16);
+#else
     crypto_auth(auth, data, length, key);
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
 bool crypto_hmac_verify(const uint8_t auth[CRYPTO_HMAC_SIZE], const uint8_t key[CRYPTO_HMAC_KEY_SIZE],
                         const uint8_t *data, size_t length)
 {
+#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+    return memcmp(auth, key, 16) == 0 && memcmp(auth + 16, data, length < 16 ? length : 16) == 0;
+#else
     return crypto_auth_verify(auth, data, length, key) == 0;
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-void crypto_sha256(uint8_t *hash, const uint8_t *data, size_t length)
+void crypto_sha256(uint8_t hash[CRYPTO_SHA256_SIZE], const uint8_t *data, size_t length)
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
-    memset(hash, 0, CRYPTO_SHA256_SIZE);
+    memzero(hash, CRYPTO_SHA256_SIZE);
     memcpy(hash, data, length < CRYPTO_SHA256_SIZE ? length : CRYPTO_SHA256_SIZE);
 #else
     crypto_hash_sha256(hash, data, length);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
-void crypto_sha512(uint8_t *hash, const uint8_t *data, size_t length)
+void crypto_sha512(uint8_t hash[CRYPTO_SHA512_SIZE], const uint8_t *data, size_t length)
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
-    memset(hash, 0, CRYPTO_SHA512_SIZE);
+    memzero(hash, CRYPTO_SHA512_SIZE);
     memcpy(hash, data, length < CRYPTO_SHA512_SIZE ? length : CRYPTO_SHA512_SIZE);
 #else
     crypto_hash_sha512(hash, data, length);
-#endif
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 }
 
 non_null()
@@ -530,41 +499,29 @@ static void sys_random_bytes(void *obj, uint8_t *bytes, size_t length)
 non_null()
 static uint32_t sys_random_uniform(void *obj, uint32_t upper_bound)
 {
-#ifdef VANILLA_NACL
-    if (upper_bound == 0) {
-        return 0;
-    }
-
-    uint32_t randnum;
-    sys_random_bytes(obj, (uint8_t *)&randnum, sizeof(randnum));
-    return randnum % upper_bound;
-#else
     return randombytes_uniform(upper_bound);
-#endif
 }
 
-static const Random_Funcs system_random_funcs = {
+static const Random_Funcs os_random_funcs = {
     sys_random_bytes,
     sys_random_uniform,
 };
 
-static const Random system_random_obj = {&system_random_funcs};
+static const Random os_random_obj = {&os_random_funcs};
 
-const Random *system_random(void)
+const Random *os_random(void)
 {
 #ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     if ((true)) {
         return nullptr;
     }
-#endif
-#ifndef VANILLA_NACL
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
     // It is safe to call this function more than once and from different
     // threads -- subsequent calls won't have any effects.
     if (sodium_init() == -1) {
         return nullptr;
     }
-#endif
-    return &system_random_obj;
+    return &os_random_obj;
 }
 
 void random_bytes(const Random *rng, uint8_t *bytes, size_t length)
