@@ -8,13 +8,13 @@
  */
 #include "ping_array.h"
 
-#include <stdlib.h>
 #include <string.h>
 
+#include "attributes.h"
 #include "ccompat.h"
 #include "crypto_core.h"
+#include "mem.h"
 #include "mono_time.h"
-#include "util.h"
 
 typedef struct Ping_Array_Entry {
     uint8_t *data;
@@ -24,6 +24,7 @@ typedef struct Ping_Array_Entry {
 } Ping_Array_Entry;
 
 struct Ping_Array {
+    const Memory *mem;
     Ping_Array_Entry *entries;
 
     uint32_t last_deleted; /* number representing the next entry to be deleted. */
@@ -32,7 +33,7 @@ struct Ping_Array {
     uint32_t timeout;      /* The timeout after which entries are cleared. */
 };
 
-Ping_Array *ping_array_new(uint32_t size, uint32_t timeout)
+Ping_Array *ping_array_new(const Memory *mem, uint32_t size, uint32_t timeout)
 {
     if (size == 0 || timeout == 0) {
         return nullptr;
@@ -43,19 +44,21 @@ Ping_Array *ping_array_new(uint32_t size, uint32_t timeout)
         return nullptr;
     }
 
-    Ping_Array *const empty_array = (Ping_Array *)calloc(1, sizeof(Ping_Array));
+    Ping_Array *const empty_array = (Ping_Array *)mem_alloc(mem, sizeof(Ping_Array));
 
     if (empty_array == nullptr) {
         return nullptr;
     }
 
-    empty_array->entries = (Ping_Array_Entry *)calloc(size, sizeof(Ping_Array_Entry));
+    Ping_Array_Entry *entries = (Ping_Array_Entry *)mem_valloc(mem, size, sizeof(Ping_Array_Entry));
 
-    if (empty_array->entries == nullptr) {
-        free(empty_array);
+    if (entries == nullptr) {
+        mem_delete(mem, empty_array);
         return nullptr;
     }
 
+    empty_array->mem = mem;
+    empty_array->entries = entries;
     empty_array->last_deleted = 0;
     empty_array->last_added = 0;
     empty_array->total_size = size;
@@ -67,7 +70,7 @@ non_null()
 static void clear_entry(Ping_Array *array, uint32_t index)
 {
     const Ping_Array_Entry empty = {nullptr};
-    free(array->entries[index].data);
+    mem_delete(array->mem, array->entries[index].data);
     array->entries[index] = empty;
 }
 
@@ -83,8 +86,8 @@ void ping_array_kill(Ping_Array *array)
         ++array->last_deleted;
     }
 
-    free(array->entries);
-    free(array);
+    mem_delete(array->mem, array->entries);
+    mem_delete(array->mem, array);
 }
 
 /** Clear timed out entries. */
@@ -114,13 +117,16 @@ uint64_t ping_array_add(Ping_Array *array, const Mono_Time *mono_time, const Ran
         clear_entry(array, index);
     }
 
-    array->entries[index].data = (uint8_t *)malloc(length);
+    uint8_t *entry_data = (uint8_t *)mem_balloc(array->mem, length);
 
-    if (array->entries[index].data == nullptr) {
+    if (entry_data == nullptr) {
+        array->entries[index].data = nullptr;
         return 0;
     }
 
-    memcpy(array->entries[index].data, data, length);
+    memcpy(entry_data, data, length);
+
+    array->entries[index].data = entry_data;
     array->entries[index].length = length;
     array->entries[index].ping_time = mono_time_get(mono_time);
     ++array->last_added;
