@@ -3,13 +3,13 @@
 #include <gtest/gtest.h>
 
 #include "crypto_core.h"
+#include "crypto_core_test_util.hh"
 
 namespace {
 
 TEST(Util, TwoRandomIdsAreNotEqual)
 {
-    const Random *rng = system_random();
-    ASSERT_NE(rng, nullptr);
+    Test_Random rng;
     uint8_t pk1[CRYPTO_PUBLIC_KEY_SIZE];
     uint8_t sk1[CRYPTO_SECRET_KEY_SIZE];
     uint8_t pk2[CRYPTO_PUBLIC_KEY_SIZE];
@@ -23,8 +23,7 @@ TEST(Util, TwoRandomIdsAreNotEqual)
 
 TEST(Util, IdCopyMakesKeysEqual)
 {
-    const Random *rng = system_random();
-    ASSERT_NE(rng, nullptr);
+    Test_Random rng;
     uint8_t pk1[CRYPTO_PUBLIC_KEY_SIZE];
     uint8_t sk1[CRYPTO_SECRET_KEY_SIZE];
     uint8_t pk2[CRYPTO_PUBLIC_KEY_SIZE] = {0};
@@ -35,4 +34,15 @@ TEST(Util, IdCopyMakesKeysEqual)
     EXPECT_TRUE(pk_equal(pk1, pk2));
 }
 
+TEST(Cmp, OrdersNumbersCorrectly)
+{
+    EXPECT_EQ(cmp_uint(1, 2), -1);
+    EXPECT_EQ(cmp_uint(0, UINT32_MAX), -1);
+    EXPECT_EQ(cmp_uint(UINT32_MAX, 0), 1);
+    EXPECT_EQ(cmp_uint(UINT32_MAX, UINT32_MAX), 0);
+    EXPECT_EQ(cmp_uint(0, UINT64_MAX), -1);
+    EXPECT_EQ(cmp_uint(UINT64_MAX, 0), 1);
+    EXPECT_EQ(cmp_uint(UINT64_MAX, UINT64_MAX), 0);
+}
+
 }  // namespace
